==> audiochain-1.1.5-src.tgz: org/audiochain/io/SignalGeneratorAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static java.lang.Math.*;
import static java.util.Arrays.*;

import java.util.Random;

public class SignalGeneratorAudioDataReader implements AudioDataReader {

	public static enum SignalType {
		Sine, Saw, Triangle, Noise, Silence
	}

	private final int sampleMaximum; // full scale ie 32767 in the case of 16
										// bit sample size
	private final float frameRate;
	private final int samplePeakToPeakAmplitudeSize;
	private static final double twopi = PI * 2;
	private SignalType signalType = SignalType.Sine;
	private float frequency = 1000F;
	private double cosineIncrement;
	private float linearSampleIncrement;
	private float amplitude = 1F;
	private float level = 0F;
	private long dt; // discret time position, moves along the x-Axis for Sine
	private boolean stopped;
	private int sample;
	private boolean incrementTriangle;
	private Random random;

	public SignalGeneratorAudioDataReader(int sampleSize, float frameRate) {
		this(sampleSize, frameRate, SignalType.Sine);
	}

	public SignalGeneratorAudioDataReader(int sampleSize, float frameRate, SignalType signalType) {
		this.sampleMaximum = (2 << sampleSize - 2) - 1; // ie 32767
		this.samplePeakToPeakAmplitudeSize = (2 << sampleSize - 1) - 1; // ie
																		// 65535
		this.frameRate = frameRate;
		this.random = new Random();
		this.signalType = signalType;
	}

	@Override
	public int read(int[] data) {
		if (stopped) {
			return -1;
		}
		int len = data.length;

		double amp = amplitude * sampleMaximum;
		int peakToPeak = (int) (amplitude * samplePeakToPeakAmplitudeSize);
		switch (signalType) {
		case Sine:
			sine(data, amp);
			break;
		case Saw:
			saw(data, amp);
			break;
		case Triangle:
			triangle(data, amp);
			break;
		case Noise:
			noise(data, amp, peakToPeak);
			break;
		case Silence:
		default:
			fill(data, 0);
			break;
		}

		return len;
	}

	private void noise(int[] data, double amp, int peakToPeak) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			// generates values between -peak and +peak (ie: -32768 and
			// +32767)
			if (peakToPeak == 0) {
				sample = 0;
			} else {
				sample = random.nextInt(peakToPeak) - (int) amp - 1;
			}
			data[l] = sample;
			data[r] = sample;
		}
	}

	private void triangle(int[] data, double amp) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			if (incrementTriangle) {
				sample += linearSampleIncrement;
			} else {
				sample -= linearSampleIncrement;
			}
			if (sample >= (int) amp) {
				incrementTriangle = false;
			} else if (sample <= (int) -amp) {
				incrementTriangle = true;
			}
			data[l] = sample;
			data[r] = sample;
		}
	}

	private void saw(int[] data, double amp) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			sample += linearSampleIncrement;
			if (sample >= (int) amp) {
				sample = (int) -amp;
			}
			data[l] = sample;
			data[r] = sample;
		}
	}

	private void sine(int[] data, double amp) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			sample = (int) (amp * cos(cosineIncrement * dt++));
			data[l] = sample;
			data[r] = sample;
		}
	}

	@Override
	public void close() {

	}

	@Override
	public void stop() {
		stopped = true;
	}

	public SignalType getSignalType() {
		return signalType;
	}

	public void setSignalType(SignalType signalType) {
		this.signalType = signalType;
	}

	public float getFrequency() {
		return frequency;
	}

	public void setFrequency(float frequency) {
		if (frequency < 0) {
			frequency = 0;
		}
		this.frequency = frequency;
		cosineIncrement = twopi * frequency / frameRate;
		float framesPerCycle = frameRate / frequency;
		linearSampleIncrement = sampleMaximum / framesPerCycle;
	}

	public float getAmplitude() {
		return amplitude;
	}

	/**
	 * @param amplitude
	 *            as factor between 0 and 1.
	 */
	public void setAmplitude(float amplitude) {
		this.amplitude = amplitude;
		this.level = 20F * (float) Math.log10(amplitude);
	}

	/**
	 * @param level
	 *            as decibel
	 */
	public void setLevel(float level) {
		this.level = level;
		// level in dB to factor
		float exp = level / 20F;
		this.amplitude = (float) Math.pow(10D, exp);
	}

	public float getSampleMaximum() {
		return sampleMaximum;
	}

	public float getLevel() {
		return level;
	}

	@Override
	public void seek(long samplePosition) {
		// nop
	}

	public static double[] createSin(double[] x, double amplitude, double frequency) {
		int n = x.length;
		double k = frequency * twopi / n;
		for (int i = 0; i < n; i++) {
			x[i] = amplitude * sin(k * i);
		}
		return x;
	}

	public static double[] createCos(double[] x, double amplitude, double frequency) {
		int n = x.length;
		double k = frequency * twopi / n;
		for (int i = 0; i < n; i++) {
			x[i] = amplitude * cos(k * i);
		}
		return x;
	}

	public static double[] createNoise(int n) {
		double[] random = new double[n];
		for (int i = 0; i < n; i++) {
			random[i] = random() * 2 - 1;
		}
		return random;
	}

	public static int[] createTestSine(float frequency) {
		final int sampleSize = 16;
		float sampleRate = 44100;
		SignalGeneratorAudioDataReader signal = new SignalGeneratorAudioDataReader(sampleSize, sampleRate);
		signal.setFrequency(frequency);
		signal.setSignalType(SignalType.Sine);

		int[] sin = new int[4410];
		signal.read(sin);
		int[] testdata = new int[88200];

		System.arraycopy(sin, 0, testdata, 0, sin.length);
		return testdata;
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/io/SignalGeneratorAudioDataReader.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.io;

import static java.lang.Math.PI;
import static java.lang.Math.cos;
import static java.lang.Math.random;
import static java.lang.Math.sin;
import static java.util.Arrays.fill;

import java.util.Random;

public class SignalGeneratorAudioDataReader implements AudioDataReader {

	public static enum SignalType {
		Sine, Saw, Triangle, Noise, Silence
	}

	private final int sampleMaximum; // full scale ie 32767 in the case of 16
										// bit sample size
	private final float frameRate;
	private final int samplePeakToPeakAmplitudeSize;
	private static final double twopi = PI * 2;
	private SignalType signalType = SignalType.Sine;
	private float frequency = 1000F;
	private double cosineIncrement;
	private float linearSampleIncrement;
	private float amplitude = 1F;
	private float level = 0F;
	private long dt; // discret time position, moves along the x-Axis for Sine
	private boolean stopped;
	private int sample;
	private boolean incrementTriangle;
	private Random random;

	public SignalGeneratorAudioDataReader(int sampleSize, float frameRate) {
		this(sampleSize, frameRate, SignalType.Sine);
	}

	public SignalGeneratorAudioDataReader(int sampleSize, float frameRate, SignalType signalType) {
		this.sampleMaximum = (2 << sampleSize - 2) - 1; // ie 32767
		this.samplePeakToPeakAmplitudeSize = (2 << sampleSize - 1) - 1; // ie
																		// 65535
		this.frameRate = frameRate;
		this.random = new Random();
		this.signalType = signalType;
	}

	@Override
	public int read(int[] data) {
		if (stopped) {
			return -1;
		}
		int len = data.length;

		double amp = amplitude * sampleMaximum;
		int peakToPeak = (int) (amplitude * samplePeakToPeakAmplitudeSize);
		switch (signalType) {
		case Sine:
			sine(data, amp);
			break;
		case Saw:
			saw(data, amp);
			break;
		case Triangle:
			triangle(data, amp);
			break;
		case Noise:
			noise(data, amp, peakToPeak);
			break;
		case Silence:
		default:
			fill(data, 0);
			break;
		}

		return len;
	}

	private void noise(int[] data, double amp, int peakToPeak) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			// generates values between -peak and +peak (ie: -32768 and
			// +32767)
			if (peakToPeak == 0) {
				sample = 0;
			} else {
				sample = random.nextInt(peakToPeak) - (int) amp - 1;
			}
			data[l] = sample;
			data[r] = sample;
		}
	}

	private void triangle(int[] data, double amp) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			if (incrementTriangle) {
				sample += linearSampleIncrement;
			} else {
				sample -= linearSampleIncrement;
			}
			if (sample >= (int) amp) {
				incrementTriangle = false;
			} else if (sample <= (int) -amp) {
				incrementTriangle = true;
			}
			data[l] = sample;
			data[r] = sample;
		}
	}

	private void saw(int[] data, double amp) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			sample += linearSampleIncrement;
			if (sample >= (int) amp) {
				sample = (int) -amp;
			}
			data[l] = sample;
			data[r] = sample;
		}
	}

	private void sine(int[] data, double amp) {
		for (int l = 0, r = 1, n = data.length; l < n; l += 2, r += 2) {
			sample = (int) (amp * cos(cosineIncrement * dt++));
			data[l] = sample;
			data[r] = sample;
		}
	}

	@Override
	public void close() {

	}

	@Override
	public void stop() {
		stopped = true;
	}

	public SignalType getSignalType() {
		return signalType;
	}

	public void setSignalType(SignalType signalType) {
		this.signalType = signalType;
	}

	public float getFrequency() {
		return frequency;
	}

	public void setFrequency(float frequency) {
		if (frequency < 0) {
			frequency = 0;
		}
		this.frequency = frequency;
		cosineIncrement = twopi * frequency / frameRate;
		float framesPerCycle = frameRate / frequency;
		linearSampleIncrement = sampleMaximum / framesPerCycle;
	}

	public float getAmplitude() {
		return amplitude;
	}

	/**
	 * @param amplitude
	 *            as factor between 0 and 1.
	 */
	public void setAmplitude(float amplitude) {
		this.amplitude = amplitude;
		this.level = 20F * (float) Math.log10(amplitude);
	}

	/**
	 * @param level
	 *            as decibel
	 */
	public void setLevel(float level) {
		this.level = level;
		// level in dB to factor
		float exp = level / 20F;
		this.amplitude = (float) Math.pow(10D, exp);
	}

	public float getSampleMaximum() {
		return sampleMaximum;
	}

	public float getLevel() {
		return level;
	}

	@Override
	public void seek(long samplePosition) {
		// nop
	}

	public static double[] createSin(double[] x, double amplitude, double frequency) {
		int n = x.length;
		double k = frequency * twopi / n;
		for (int i = 0; i < n; i++) {
			x[i] = amplitude * sin(k * i);
		}
		return x;
	}

	public static double[] createCos(double[] x, double amplitude, double frequency) {
		int n = x.length;
		double k = frequency * twopi / n;
		for (int i = 0; i < n; i++) {
			x[i] = amplitude * cos(k * i);
		}
		return x;
	}

	public static double[] createNoise(int n) {
		double[] random = new double[n];
		for (int i = 0; i < n; i++) {
			random[i] = random() * 2 - 1;
		}
		return random;
	}

	public static int[] createTestSine(float frequency) {
		final int sampleSize = 16;
		float sampleRate = 44100;
		SignalGeneratorAudioDataReader signal = new SignalGeneratorAudioDataReader(sampleSize, sampleRate);
		signal.setFrequency(frequency);
		signal.setSignalType(SignalType.Sine);

		int[] sin = new int[4410];
		signal.read(sin);
		int[] testdata = new int[88200];

		System.arraycopy(sin, 0, testdata, 0, sin.length);
		signal.close();
		return testdata;
	}
}
